{-# LANGUAGE DataKinds #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE TypeOperators #-}
{-# LANGUAGE OverloadedStrings #-}
{-# LANGUAGE TemplateHaskell #-}

module ISIDORE.Client where

import Servant.API
import Servant.Client

import Data.Proxy(Proxy(..))
import Data.ByteString.Lazy (ByteString)
import Data.Aeson
import qualified Data.Text as T

data LocalContent = LocalContent
  {
    _lang :: T.Text,
    _content :: T.Text
  }
  deriving (Show)

instance FromJSON LocalContent where
  parseJSON (Object o) = LocalContent <$>
                         (o .: "@xml:lang")
                         <*> (o .: "$")

data Author = Author
  {
    _firstName :: T.Text,
    _lastName :: T.Text
  }
  deriving (Show)

instance FromJSON Author where
  parseJSON (Object o) = Author <$>
                         (o .: "firstName")
                         <*> (o .: "lastName")

data MayBeLocal = Local LocalContent | NonLocal T.Text
  deriving (Show)


data IsidoreDoc = IsidoreDoc
  {
    _title :: MayBeLocal,
    _abstract :: MayBeLocal,
    _authors :: [Author]
  }
  deriving (Show)

instance FromJSON IsidoreDoc where
  parseJSON (Object o) = IsidoreDoc <$>
                         (responseReplies >>= (.: "title") )
                         <*> (responseReplies >>= (.: "abstract"))
                         <*> (responseReplies >>= (.: "enrichedCreators") >>= (.: "creator"))
    where responseReplies = (o .: "response")
            >>= (.: "replies")
            >>= (.: "content")
            >>= (.: "reply")
            >>= (.: "isidore")

data Output = JSON

instance ToHttpApiData Output where
  toUrlPiece JSON = "json"

type ISIDOREAPI = Search

-- search?q=colza&output=json&replies=1&author=jm&
type Search = "search"
  :> QueryParam "replies" Int
  :> QueryParam "output" Output
  :> QueryParam "q" T.Text
  :> QueryParam "author" T.Text
  :> Get '[JSON] T.Text

isidoreAPI :: Proxy ISIDOREAPI
isidoreAPI = Proxy

search :: Maybe Int -> Maybe Output -> Maybe T.Text -> Maybe T.Text -> ClientM T.Text

search = client isidoreAPI
